package cn.com.ty.lift.business.library.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.SelectBatchPlatformCompanyRequest;
import cn.com.ty.lift.business.library.dao.entity.model.PlatformCompanyRequest;
import cn.com.ty.lift.business.library.dao.mapper.PlatformCompanyLiftRelevanceMapper;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author bieao
 * @date 2019/12/5
 * @description 平台企业电梯关联业务层
 */
@Service
public class PlatformCompanyLiftRelevanceService extends ServiceImpl<PlatformCompanyLiftRelevanceMapper, PlatformCompanyLiftRelevance> {

    @Resource
    private PlatformCompanyLiftRelevanceMapper mapper;

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private LiftService liftService;

    @Resource
    private MaintenancePlanService maintenancePlanService;

    /**
     * @param id 电梯id
     * @param mtCompanyId 公司id
     * @return PlatformCompanyLiftRelevance
     * @description 新增平台电梯、企业电梯关联表数据
     * @date 2019/12/5 2:38 PM
     */
    public PlatformCompanyLiftRelevance save(Long id, Long mtCompanyId) {
        //建立企业电梯关联关系
        PlatformCompanyLiftRelevance companyLiftEntry = new PlatformCompanyLiftRelevance();
        companyLiftEntry.setLiftId(id);
        companyLiftEntry.setCompanyId(mtCompanyId);
        if (save(companyLiftEntry)) {
            return companyLiftEntry;
        }
        return null;
    }

    /**
     * @param paramMap 电梯id，公司id
     * @return 是否存在
     * @description 查询该公司下是否有此电梯
     * @date 2019/12/20 11:39 AM
     */
    public boolean findCompanyLiftExist(Map<String,Object> paramMap) {
        return mapper.findCompanyLiftExist(paramMap) > 0;
    }

    /**
     * @param id 公司ID
     * @return 可用电梯台量
     * @description 查询维保公司下的最多可用电梯台量
     * @date 2019/12/12 5:16 PM
     */
    public Integer countLiftNum(Long id) {
        return mapper.count(id);
    }

    /**
     * @param request liftIdList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量查询电梯状态
     * @date 2019/12/5 5:24 PM
     */
    public RestResponse selectBatch(SelectBatchPlatformCompanyRequest request) {
        List<PlatformCompanyLiftRelevance> list = mapper.selectBatchIds(request.getLiftIdList());
        if (ObjectUtil.isEmpty(list)) {
            return RestResponse.success();
        }
        return RestResponse.success(list, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request liftIdList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 服务操作
     * @date 2019/12/10 5:42 PM
     */
    public RestResponse serviceOperate(SelectBatchPlatformCompanyRequest request) {
        String operate = request.getOperate();
        switch (operate) {
            //恢复服务
            case "1":
                boolean judgeResult = liftService.judge(request.getMtCompanyId());
                if (!judgeResult) {
                    return RestResponse.fail(MessageUtils.get("msg.limit.num"));
                }
                break;
            //停止服务
            case "2":
                //TODO 检查电梯是否有进行中的任务，并提醒，批量停保项目下的电梯，清除维保、年检、急修任务
                List<PlatformCompanyLiftRelevance> resultList = mapper.selectBatchIds(request.getLiftIdList());
                //项目下非停保状态的电梯状态修改为正常
                for (PlatformCompanyLiftRelevance entry : resultList) {
                    entry.setLiftCompanyStatus(CommonEnum.LiftStatus.STOP_INSURANCE.getCode());
                }
                boolean ret = updateBatchById(resultList);
                if (!ret) {
                    return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
                }
                break;
            //继续服务
            case "3":
                Project project = new Project();
                project.setId(request.getProjectId());
                project.setProjectStatus(new Byte(CommonEnum.ProjectStatus.IN_SERVICE.getCode()));
                int result = projectMapper.updateById(project);
                if (result < 0) {
                    return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
                }
                break;
            default:
                break;
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param request platformCompanyId 公司电梯关联表主键id
     * @param request companyId 公司id
     * @param request liftId 电梯id
     * @return 1.成功, 0.失败, 消息描述
     * @description 更新电梯状态为停保,并清除维保计划
     * @date 2019/12/5 5:24 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse stopInsurance(PlatformCompanyRequest request) {
        PlatformCompanyLiftRelevance entry = new PlatformCompanyLiftRelevance();
        entry.setId(request.getPlatformCompanyId());
        entry.setLiftCompanyStatus(CommonEnum.LiftStatus.STOP_INSURANCE.getCode());
        boolean result = updateById(entry);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
        }
        //清除维保计划
        boolean ret = maintenancePlanService.cleanPlan(request.getCompanyId(), request.getLiftId());
        if (!ret) {
            return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.batch.success"));
    }

    /**
     * @param request companyIdList 公司电梯关联表主键id列表
     * @param request companyId 公司id
     * @param request idList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量更新电梯状态为停保,并清除维保计划
     * @date 2019/12/31 9:54 AM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse batchStopInsurance(PlatformCompanyRequest request){
        //公司电梯关联表主键id列表
        List<Long> companyIdList = request.getPlatformCompanyIdList();
        //批量查询公司下电梯数据
        List<PlatformCompanyLiftRelevance> list = mapper.selectBatchIds(companyIdList);
        for (PlatformCompanyLiftRelevance entry : list) {
            entry.setLiftCompanyStatus(CommonEnum.LiftStatus.STOP_INSURANCE.getCode());
        }
        //批量更新公司下电梯信息为停保
        boolean result = updateBatchById(list);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
        }
        //批量清除维保计划
        boolean ret = maintenancePlanService.removeMaintenancePlan(request.getCompanyId(),request.getIdList());
        if (!ret) {
            return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.batch.success"));
    }
}
