package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.BatchUpdateLiftRequest;
import cn.com.ty.lift.business.library.dao.entity.model.LiftExtensionRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectLiftRelevanceRequest;
import cn.com.ty.lift.business.project.dao.mapper.ProjectLiftRelevanceMapper;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectRelevanceResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author bieao
 * @date 2019/12/5
 * @description 项目电梯关联业务层
 */
@Service
public class ProjectLiftRelevanceService extends ServiceImpl<ProjectLiftRelevanceMapper, ProjectLiftRelevance> {

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private ProjectLiftRelevanceMapper projectLiftRelevanceMapper;
    /**
     * @param entry 项目关联数据
     * @return RestResponse 状态码和返回消息
     * @description 逻辑删除电梯
     * @date 2019/12/5 2:16 PM
     */
    public RestResponse logicDelete(ProjectLiftRelevance entry) {
        boolean result = updateById(entry);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request 电梯扩展请求
     * @param id        平台企业电梯主键id
     * @param workerId  维保工id
     * @return projectLiftEntry
     * @description 新增项目电梯关联数据
     * @date 2019/12/5 2:39 PM
     */
    public boolean save(LiftExtensionRequest request, long id, long workerId) {
        Lift lift = request.getLift();
        //新增项目电梯关联表
        ProjectLiftRelevance projectLiftEntry = new ProjectLiftRelevance();
        projectLiftEntry.setProjectId(request.getProjectId());
        projectLiftEntry.setLiftId(lift.getId());
        projectLiftEntry.setRelevanceId(id);
        projectLiftEntry.setWorkerId(workerId);
        projectLiftEntry.setCompanyId(request.getMtCompanyId());
        return save(projectLiftEntry);
    }


    /**
     * @param companyId 公司id
     * @param idList 电梯id列表
     * @return 项目电梯关联列表
     * @description 根据公司id和电梯id列表查询项目电梯关联列表
     * @date 2019/12/28 1:55 PM
     */
    public List<ProjectLiftRelevance> list(String companyId, List<Long> idList){
        QueryWrapper<ProjectLiftRelevance> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mt_company_id", companyId);
        queryWrapper.in("lift_id", idList);
        return list();
    }

    /**
     * @param request 电梯id列表
     * @return RestResponse 状态码和返回消息
     * @description 电梯锁定、解锁
     * @date 2019/12/5 11:00 AM
     */
    public RestResponse liftLockOrUnLock(BatchUpdateLiftRequest request) {
        List<Long> list = request.getLiftList();
        List<ProjectLiftRelevance> projectLiftList = list(request.getMtCompanyId(), list);
        if (ObjectUtil.isNotEmpty(projectLiftList))
            projectLiftList.forEach(entry -> entry.setLiftLocked(request.getLock()));
        boolean result = updateBatchById(projectLiftList);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request projectId 项目id
     * @return RestResponse 状态码和返回消息
     * @description 项目锁定、解锁,锁定该项目下的电梯
     * @date 2019/12/27 5:04 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse projectLockOrUnLock (BatchUpdateLiftRequest request){
        Project project = new Project();
        project.setId(request.getProjectId());
        project.setLocked(request.getLock());
        int ret = projectMapper.updateById(project);
        if (ret < 0){
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        //根据项目id和公司id查询项目下电梯
        List<ProjectLiftRelevance> relevanceList = projectLiftRelevanceMapper.findIdList(request);
        relevanceList.forEach(relevance -> relevance.setLiftLocked(request.getLock()));
        boolean response = updateBatchById(relevanceList);
        if (!response){
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request 用户id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询用户下的关联项目和电梯信息
     * @date 2019/12/26 4:27 PM
     */
    public IPage<ProjectRelevanceResponse> relevanceList(ProjectLiftRelevanceRequest request){
        IPage<ProjectRelevanceResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return projectLiftRelevanceMapper.selectListByUserId(page, request);
    }
}
