package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.entity.MaintenanceCompany;
import cn.com.ty.lift.enterprise.oa.service.AttendanceService;
import cn.com.ty.lift.enterprise.oa.service.MaintenanceCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
 * 打卡签到记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/oa/attendance")
public class AttendanceController {

    private AttendanceService attendanceService;
    private MaintenanceCompanyService maintenanceCompanyService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 9:44
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody AttendanceRequest request) {
        Long id = request.getId();
        if(null == id || id <= 0){
            return RestResponse.failParam();
        }
        Attendance entity = attendanceService.getById(id);
        if(null == entity){
            return RestResponse.fail();
        }
        return RestResponse.success(entity);
    }

    /**
     * @description 根据条件查询
     * @date 2019/12/6 9:44
     * @param
     * @return
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody AttendanceRequest request){
        String name = request.getUsername();
        if(StrUtil.isNotBlank(name)){
            request.setUsername(StrUtil.format("%{}%",name));
        }

        IPage<AttendanceResponse> pages = attendanceService.selectByCondition(request);
        if(null == pages){
            return RestResponse.fail();
        }
        long count = attendanceService.count();
        CountPage<AttendanceResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 设置打卡时间
     * @date 2019/12/20 16:30
     * @param
     * @return
     */
    @PostMapping("setupTime")
    public RestResponse setupTime(@RequestBody AttendanceRequest request){
        LocalDateTime topTime = request.getTopTime();
        LocalDateTime downTime = request.getDownTime();
        LocalDateTime topTimePm = request.getTopTimePm();
        LocalDateTime downTimePm = request.getDownTimePm();
        LocalDateTime topTimeOt = request.getTopTimeOt();
        LocalDateTime downTimeOt = request.getDownTimeOt();
        if(topTime != null && downTime == null){
            return RestResponse.fail("请设置上午的上下班时间");
        }
        if(topTimePm !=null && downTimePm == null){
            return RestResponse.fail("请设置下午的上下班时间");
        }
        if(topTimeOt != null && downTimeOt == null){
            return RestResponse.fail("请设置加班的上下班时间");
        }
        long mtCompanyId = request.getMtCompanyId();
        MaintenanceCompany entity = maintenanceCompanyService.getById(mtCompanyId);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setTopTime(topTime);
        entity.setDownTime(downTime);
        entity.setTopTimePm(topTimePm);
        entity.setDownTimePm(downTimePm);
        entity.setTopTimeOt(topTimeOt);
        entity.setDownTimeOt(downTimeOt);
        boolean result = maintenanceCompanyService.saveOrUpdate(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }
}
